/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#ifndef _guitareditor_
#define _guitareditor_

#include "Editors/Editor.h"

#include <vector>

#include "Editors/RelativeXCoord.h"

namespace AriaMaestosa {
	
class Track; // forward
class TuningPicker;

class GuitarEditor : public Editor {
    
    bool dragging_on_track;
    bool clickedOnNote;
    int lastClickedNote; // contains the ID of the latest clicked note, or -1 to mean "selected notes"
    TuningPicker* tuningPicker;
	
public:
    
    std::vector<int> tuning;
    
    GuitarEditor(Track* track);
    
    void render(int mousex_current=-1, int mousey_current=-1, int mousex_initial=-1, int mousey_initial=-1, bool focus=false);
    
    void tuningUpdated();
    void updatePosition(const int from_y, const int to_y, const int width, const int height, const int barHeight);
    
	void selectNotesInRect(RelativeXCoord<int>& mousex_current, int mousey_current, RelativeXCoord<int>& mousex_initial, int mousey_initial);
	
    void mouseDown(const int x, const int y);
    void mouseDrag(const int mousex_current, const int mousey_current, const int mousex_initial, const int mousey_initial);
    void mouseUp(const int mousex_current, const int mousey_current, const int mousex_initial, const int mousey_initial);
    void rightClick(const int x, const int y);
    void mouseExited(int dragX_arg, int mousey_current, int XBeforeDrag_arg,int mousey_initial);
		
	void rearrangeNotes(); // rearrange notes smartly (chooses on which string each note should go)
	
    int getYScrollInPixels();
    
    ~GuitarEditor();
};

}

#endif
